﻿using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using SampleExternalServiceAPI.BL;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Security.Cryptography;
using System.Text;
using System.Threading;
using System.Web;
using System.Web.Http;

namespace SampleExternalServiceAPI.Controllers
{
    public class TaxController : ApiController
    {
        /// <summary>
        /// The method calculate tax based on calculationDetails input 
        /// </summary>
        /// <param name="calculationDetails"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("tax/calculate")]
        public HttpResponseMessage CalculateTax([FromBody] JObject calculationDetails)
        {
            try
            {
                Dictionary<string, string> queryString = Request.GetQueryNameValuePairs().ToDictionary(x => x.Key, x => x.Value);

                int caseVal = queryString.Keys.Contains("case") ? int.Parse(queryString["case"]) : -1;
                decimal taxPercentage = queryString.Keys.Contains("taxPercentage") ? decimal.Parse(queryString["taxPercentage"]) : 0.2m;
                int timeOutSeconds = queryString.Keys.Contains("timeOutSeconds") ? int.Parse(queryString["timeOutSeconds"]) : 0;
                bool validateManufacturer = queryString.Keys.Contains("validateManufacturer") ? bool.Parse(queryString["validateManufacturer"]) : false;
                bool validatePickupAddress = queryString.Keys.Contains("validatePickupAddress") ? bool.Parse(queryString["validatePickupAddress"]) : false;
                bool validateTaxCode = queryString.Keys.Contains("validateTaxCode") ? bool.Parse(queryString["validateTaxCode"]) : false;


                string jsonOut = string.Empty;
                string hashResult = string.Empty;
                TaxCalculatorManager taxManager = new TaxCalculatorManager(taxPercentage, validateManufacturer, 
                                                                                validatePickupAddress, validateTaxCode);
                taxManager.CalculateTax(calculationDetails);

                switch (caseVal)
                {
                    case 1:
                        jsonOut = taxManager.GetMinimalJsonResult();
                        break;
                    case 2:
                        jsonOut = taxManager.GetFullJsonResult();
                        break;
                    case 3:
                        string message = string.Format("Tax calculation was failed");
                        HttpError err = new HttpError(message);
                        return Request.CreateResponse(HttpStatusCode.BadRequest, err);
                    case 4:
                        return Request.CreateResponse(HttpStatusCode.InternalServerError);
                    case 5:
                        Thread.Sleep(timeOutSeconds * 1000);
                        jsonOut = taxManager.GetMinimalJsonResult();
                        break;
                    case 6:
                        jsonOut = taxManager.GetMinimalJsonResult();
                        hashResult = GetHash(jsonOut, "invalid");
                        break;
                    case 7:
                        jsonOut = string.Empty;
                        break;
                    default:
                        jsonOut = taxManager.GetMinimalJsonResult();
                        break;
                }

                if (string.IsNullOrEmpty(hashResult))
                {
                    hashResult = GetHash(jsonOut, Guid.Empty.ToString());
                }

                HttpResponseMessage responseMessage = new HttpResponseMessage
                {
                    StatusCode = HttpStatusCode.OK,
                    Content = new StringContent(jsonOut, Encoding.UTF8, "application/json")
                };

                responseMessage.Headers.Add("X-Signature", hashResult);

                return responseMessage;
            }
            catch (Exception err)
            {
                string message = string.Format("Tax calculation was failed- see more details in the log");
                HttpError err2 = new HttpError(err.Message);
                return Request.CreateResponse(HttpStatusCode.BadRequest, err2);
            }
            
        }

        private string GetHash(string valueStr, string key)
        {
            var valueBytes = Encoding.UTF8.GetBytes(valueStr);
            var keyBytes = Encoding.UTF8.GetBytes(key);

            var hmacsha512 = new HMACSHA512(keyBytes);
            byte[] hashBytes = hmacsha512.ComputeHash(valueBytes);

            var result = Convert.ToBase64String(hashBytes);

            return result;
        }


    }
}